@@ -78,6 +78,8 @@ struct mtpdevice_list_struct {
 };
 typedef struct mtpdevice_list_struct mtpdevice_list_t;
 
+static libusb_context *libmtp_libusb_context;
+
 struct ptp_event_cb_data {
   PTPEventCbFn cb;
   void *user_data;
@@ -153,7 +155,7 @@ static LIBMTP_error_number_t init_usb()
   if (libusb1_initialized)
      return LIBMTP_ERROR_NONE;
 
-  if (libusb_init(NULL) < 0) {
+  if (libusb_init(&libmtp_libusb_context) < 0) {
     LIBMTP_ERROR("Libusb1 init failed\n");
     return LIBMTP_ERROR_USB_LAYER;
   }
@@ -161,7 +163,8 @@ static LIBMTP_error_number_t init_usb()
   libusb1_initialized = 1;
 
   if ((LIBMTP_debug & LIBMTP_DEBUG_USB) != 0)
-    libusb_set_debug(NULL,9);
+    /*libusb_set_debug(libmtp_libusb_context,9);*/
+    libusb_set_option(libmtp_libusb_context, LIBUSB_OPTION_LOG_LEVEL,  LIBUSB_LOG_LEVEL_DEBUG ); /* highest level */
   return LIBMTP_ERROR_NONE;
 }
 
@@ -523,7 +526,7 @@ static LIBMTP_error_number_t get_mtp_usb_device_list(mtpdevice_list_t ** mtp_dev
   if (init_usb_ret != LIBMTP_ERROR_NONE)
     return init_usb_ret;
 
-  nrofdevs = libusb_get_device_list (NULL, &devs);
+  nrofdevs = libusb_get_device_list (libmtp_libusb_context, &devs);
   for (i = 0; i < nrofdevs ; i++) {
       libusb_device *dev = devs[i];
       struct libusb_device_descriptor desc;
@@ -600,7 +603,7 @@ int LIBMTP_Check_Specific_Device(int busno, int devno)
   if (init_usb_ret != LIBMTP_ERROR_NONE)
     return 0;
 
-  nrofdevs = libusb_get_device_list (NULL, &devs);
+  nrofdevs = libusb_get_device_list (libmtp_libusb_context, &devs);
   for (i = 0; i < nrofdevs ; i++ ) {
     if (libusb_get_bus_number(devs[i]) != busno)
 	continue;
@@ -1453,6 +1456,10 @@ ptp_usb_getdata (PTPParams* params, PTPContainer* ptp, PTPDataHandler *handler)
 		if (putfunc_ret != PTP_RC_OK)
 			return ptp_read_cancel_func(params, ptp->Transaction_ID);
 
+		/* Nothing more left to read*/
+		if (rlen == usbdata.length)
+			return PTP_RC_OK;
+
 		  /* stuff data directly to passed data handler */
 		  while (1) {
 		    unsigned long readdata;
@@ -1835,7 +1842,7 @@ ptp_usb_event_async (PTPParams* params, PTPEventCbFn cb, void *user_data) {
  */
 int LIBMTP_Handle_Events_Timeout_Completed(struct timeval *tv, int *completed) {
 	/* Pass NULL for context as libmtp always uses the default context */
-	return libusb_handle_events_timeout_completed(NULL, tv, completed);
+	return libusb_handle_events_timeout_completed(libmtp_libusb_context, tv, completed);
 }
 
 uint16_t
@@ -1848,11 +1855,11 @@ ptp_usb_control_cancel_request (PTPParams *params, uint32_t transactionid) {
 	htod32a(&buffer[2],transactionid);
 	ret = libusb_control_transfer(ptp_usb->handle,
 			      LIBUSB_REQUEST_TYPE_CLASS | LIBUSB_RECIPIENT_INTERFACE,
-                              0x64, 0x0000, 0x0000,
+                              0x64, 0x0000, ptp_usb->interface,
 			      buffer,
 			      sizeof(buffer),
 			      ptp_usb->timeout);
-	if (ret < sizeof(buffer))
+	if (ret < (int)sizeof(buffer))
 		return PTP_ERROR_IO;
 	return PTP_RC_OK;
 }
@@ -1870,11 +1877,11 @@ ptp_usb_control_device_status_request (PTPParams *params) {
 
     ret = libusb_control_transfer(ptp_usb->handle,
                   LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_CLASS | LIBUSB_RECIPIENT_INTERFACE,
-                  0x67, 0x0000, 0x0000,
+                  0x67, 0x0000, ptp_usb->interface,
                   buffer,
                   sizeof(buffer),
                   ptp_usb->timeout);
-    if (ret < sizeof(buffer))
+    if (ret < (int)sizeof(buffer))
         return PTP_ERROR_IO;
 
     ret = dtoh16a(&buffer[2]);
@@ -1966,7 +1973,10 @@ static int init_ptp_usb(PTPParams* params, PTP_USB* ptp_usb, libusb_device* dev)
 
   usbresult = libusb_claim_interface(device_handle, ptp_usb->interface);
   if (usbresult != 0) {
-    fprintf(stderr, "error returned by libusb_claim_interface() = %d", usbresult);
+    if (usbresult == LIBUSB_ERROR_BUSY)
+      fprintf(stderr, "libusb_claim_interface() reports device is busy, likely in use by GVFS or KDE MTP device handling already");
+    else
+      fprintf(stderr, "error returned by libusb_claim_interface() = %d", usbresult);
     return -1;
   }
 
@@ -2211,7 +2221,7 @@ LIBMTP_error_number_t configure_usb_device(LIBMTP_raw_device_t *device,
   if (init_usb_ret != LIBMTP_ERROR_NONE)
     return init_usb_ret;
 
-  nrofdevs = libusb_get_device_list(NULL, &devs);
+  nrofdevs = libusb_get_device_list(libmtp_libusb_context, &devs);
   for (i = 0; i < nrofdevs ; i++) {
     if (libusb_get_bus_number(devs[i]) != device->bus_location)
       continue;
